@@ -14,12 +14,14 @@
 #include <mapnik/version.hpp>
 #include <mapnik/value_types.hpp>
 #include <mapnik/well_known_srs.hpp>
-
+#include <mapnik/version.hpp>
 #include <mapnik/vertex.hpp>
 #include <mapnik/datasource.hpp>
 #include <mapnik/feature.hpp>
 #include <mapnik/feature_factory.hpp>
 #include <mapnik/geom_util.hpp>
+#include <mapnik/image_reader.hpp>
+#include <mapnik/raster.hpp>
 
 #include <memory>
 #include <stdexcept>
@@ -27,24 +29,79 @@
 
 #include <boost/optional.hpp>
 #include <boost/ptr_container/ptr_vector.hpp>
-#include <boost/make_shared.hpp>
-#include <boost/shared_ptr.hpp>
 #include <unicode/unistr.h>
-#include <boost/algorithm/string.hpp>
+
+#include "mapnik3x_compatibility.hpp"
+#include MAPNIK_MAKE_SHARED_INCLUDE
+#include MAPNIK_SHARED_INCLUDE
 
 namespace mapnik { namespace vector {
 
+    void add_attributes(mapnik::feature_ptr feature,
+                        mapnik::vector::tile_feature const& f,
+                        mapnik::vector::tile_layer const& layer,
+                        mapnik::transcoder const& tr)
+    {
+        std::size_t num_keys = static_cast<std::size_t>(layer.keys_size());
+        std::size_t num_values = static_cast<std::size_t>(layer.values_size());
+        for (int m = 0; m < f.tags_size(); m += 2)
+        {
+            std::size_t key_name = f.tags(m);
+            std::size_t key_value = f.tags(m + 1);
+            if (key_name < num_keys
+                && key_value < num_values)
+            {
+                std::string const& name = layer.keys(key_name);
+                if (feature->has_key(name))
+                {
+                    mapnik::vector::tile_value const& value = layer.values(key_value);
+                    if (value.has_string_value())
+                    {
+                        std::string str = value.string_value();
+                        feature->put(name, tr.transcode(str.data(), str.length()));
+                    }
+                    else if (value.has_int_value())
+                    {
+                        feature->put(name, static_cast<mapnik::value_integer>(value.int_value()));
+                    }
+                    else if (value.has_double_value())
+                    {
+                        feature->put(name, static_cast<mapnik::value_double>(value.double_value()));
+                    }
+                    else if (value.has_float_value())
+                    {
+                        feature->put(name, static_cast<mapnik::value_double>(value.float_value()));
+                    }
+                    else if (value.has_bool_value())
+                    {
+                        feature->put(name, static_cast<mapnik::value_bool>(value.bool_value()));
+                    }
+                    else if (value.has_sint_value())
+                    {
+                        feature->put(name, static_cast<mapnik::value_integer>(value.sint_value()));
+                    }
+                    else if (value.has_uint_value())
+                    {
+                        feature->put(name, static_cast<mapnik::value_integer>(value.uint_value()));
+                    }
+                }
+            }
+        }
+    }
+
     template <typename Filter>
     class tile_featureset : public Featureset
     {
     public:
         tile_featureset(Filter const& filter,
+                        mapnik::box2d<double> const& tile_extent,
                         std::set<std::string> const& attribute_names,
                         mapnik::vector::tile_layer const& layer,
                         double tile_x,
                         double tile_y,
                         double scale)
             : filter_(filter),
+              tile_extent_(tile_extent),
               layer_(layer),
               tile_x_(tile_x),
               tile_y_(tile_y),
@@ -52,7 +109,7 @@ namespace mapnik { namespace vector {
               itr_(0),
               end_(layer_.features_size()),
               tr_("utf-8"),
-              ctx_(boost::make_shared<mapnik::context_type>())
+              ctx_(MAPNIK_MAKE_SHARED<mapnik::context_type>())
         {
             std::set<std::string>::const_iterator pos = attribute_names.begin();
             std::set<std::string>::const_iterator end = attribute_names.end();
@@ -77,18 +134,43 @@ namespace mapnik { namespace vector {
             {
                 mapnik::vector::tile_feature const& f = layer_.features(itr_);
                 mapnik::value_integer feature_id = itr_++;
-                // if encoded feature was given an id, respect it
-                // https://github.com/mapbox/mapnik-vector-tile/issues/17
-                // https://github.com/mapbox/mapnik-vector-tile/issues/18
-                if (f.has_id())
+                if (f.has_raster())
                 {
-                    feature_id = f.id();
+                    std::string const& image_buffer = f.raster();
+                    MAPNIK_UNIQUE_PTR<mapnik::image_reader> reader(mapnik::get_image_reader(image_buffer.data(),image_buffer.size()));
+                    if (reader.get())
+                    {
+                        if (f.has_id())
+                        {
+                            feature_id = f.id();
+                        }
+                        #if MAPNIK_VERSION >= 300000
+                        double filter_factor = 1.0;
+                        #endif
+                        bool premultiplied = false;
+                        mapnik::feature_ptr feature = mapnik::feature_factory::create(ctx_,feature_id);
+                        mapnik::raster_ptr raster = MAPNIK_MAKE_SHARED<mapnik::raster>(
+                                    tile_extent_,
+                                    reader->width(),
+                                    reader->height(),
+                        #if MAPNIK_VERSION >= 300000
+                                    filter_factor,
+                        #endif
+                                    premultiplied
+                                    );
+                        reader->read(0,0,raster->data_);
+                        feature->set_raster(raster);
+                        add_attributes(feature,f,layer_,tr_);
+                        return feature;
+                    }
                 }
-                mapnik::feature_ptr feature(
-                    mapnik::feature_factory::create(ctx_,feature_id));
-                feature->paths().push_back(new mapnik::geometry_type(
-                        mapnik::eGeomType(f.type())));
-                mapnik::geometry_type * geom = &feature->paths().front();
+                if (f.geometry_size() <= 0)
+                {
+                    continue;
+                }
+                MAPNIK_UNIQUE_PTR<mapnik::geometry_type> geom(
+                    new mapnik::geometry_type(
+                        MAPNIK_GEOM_TYPE(f.type())));
                 int cmd = -1;
                 const int cmd_bits = 3;
                 unsigned length = 0;
@@ -116,11 +198,6 @@ namespace mapnik { namespace vector {
                             y -= (static_cast<double>(dy) / scale_);
                             if (cmd == mapnik::SEG_MOVETO)
                             {
-                                if (!first) {
-                                    feature->paths().push_back(new mapnik::geometry_type(
-                                        mapnik::eGeomType(f.type())));
-                                    geom = &feature->paths().back();
-                                }
                                 first_x = x;
                                 first_y = y;
                             }
@@ -142,7 +219,10 @@ namespace mapnik { namespace vector {
                         }
                         else
                         {
-                            throw std::runtime_error("Unknown command type");
+                            std::stringstream msg;
+                            msg << "Unknown command type (tile_featureset): "
+                                << cmd;
+                            throw std::runtime_error(msg.str());
                         }
                     }
                 }
@@ -150,63 +230,13 @@ namespace mapnik { namespace vector {
                 {
                     continue;
                 }
-
-                // attributes
-                for (int m = 0; m < f.tags_size(); m += 2)
+                if (f.has_id())
                 {
-                    std::size_t key_name = f.tags(m);
-                    std::size_t key_value = f.tags(m + 1);
-
-                    if (key_name < static_cast<std::size_t>(layer_.keys_size())
-                        && key_value < static_cast<std::size_t>(layer_.values_size()))
-                    {
-                        std::string const& name = layer_.keys(key_name);
-                        if (feature->has_key(name))
-                        {
-                            mapnik::vector::tile_value const& value = layer_.values(key_value);
-                            if (value.has_string_value())
-                            {
-                                std::string str = value.string_value();
-                                feature->put(name, tr_.transcode(str.data(), str.length()));
-                            }
-                            else if (value.has_int_value())
-                            {
-                                mapnik::value_integer val = value.int_value();
-                                feature->put(name, val);
-                            }
-                            else if (value.has_double_value())
-                            {
-                                mapnik::value_double val = value.double_value();
-                                feature->put(name, val);
-                            }
-                            else if (value.has_float_value())
-                            {
-                                mapnik::value_double val = value.float_value();
-                                feature->put(name, val);
-                            }
-                            else if (value.has_bool_value())
-                            {
-                                mapnik::value_bool val = value.bool_value();
-                                feature->put(name, val);
-                            }
-                            else if (value.has_sint_value())
-                            {
-                                mapnik::value_integer val = value.sint_value();
-                                feature->put(name, val);
-                            }
-                            else if (value.has_uint_value())
-                            {
-                                mapnik::value_integer val = value.uint_value();
-                                feature->put(name, val);
-                            }
-                            else
-                            {
-                                // Do nothing
-                                //feature->put_new(name, mapnik::value_null());
-                            }
-                        }
-                    }
+                    feature_id = f.id();
                 }
+                mapnik::feature_ptr feature = mapnik::feature_factory::create(ctx_,feature_id);
+                feature->paths().push_back(geom.release());
+                add_attributes(feature,f,layer_,tr_);
                 return feature;
             }
             return feature_ptr();
@@ -214,6 +244,7 @@ namespace mapnik { namespace vector {
 
     private:
         Filter filter_;
+        mapnik::box2d<double> tile_extent_;
         mapnik::vector::tile_layer const& layer_;
         double tile_x_;
         double tile_y_;
@@ -237,6 +268,7 @@ namespace mapnik { namespace vector {
         featureset_ptr features(query const& q) const;
         featureset_ptr features_at_point(coord2d const& pt, double tol = 0) const;
         void set_envelope(box2d<double> const& bbox);
+        box2d<double> get_tile_extent() const;
         box2d<double> envelope() const;
         boost::optional<geometry_t> get_geometry_type() const;
         layer_descriptor get_descriptor() const;
@@ -286,8 +318,8 @@ namespace mapnik { namespace vector {
     inline featureset_ptr tile_datasource::features(query const& q) const
     {
         mapnik::filter_in_box filter(q.get_bbox());
-        return boost::make_shared<tile_featureset<mapnik::filter_in_box> >
-            (filter, q.property_names(), layer_, tile_x_, tile_y_, scale_);
+        return MAPNIK_MAKE_SHARED<tile_featureset<mapnik::filter_in_box> >
+            (filter, get_tile_extent(), q.property_names(), layer_, tile_x_, tile_y_, scale_);
     }
 
     inline featureset_ptr tile_datasource::features_at_point(coord2d const& pt, double tol) const
@@ -298,8 +330,8 @@ namespace mapnik { namespace vector {
         {
             names.insert(layer_.keys(i));
         }
-        return boost::make_shared<tile_featureset<filter_at_point> >
-            (filter, names, layer_, tile_x_, tile_y_, scale_);
+        return MAPNIK_MAKE_SHARED<tile_featureset<filter_at_point> >
+            (filter, get_tile_extent(), names, layer_, tile_x_, tile_y_, scale_);
     }
 
     inline void tile_datasource::set_envelope(box2d<double> const& bbox)
@@ -308,14 +340,19 @@ namespace mapnik { namespace vector {
         extent_ = bbox;
     }
 
+    inline box2d<double> tile_datasource::get_tile_extent() const
+    {
+        mapnik::vector::spherical_mercator merc(tile_size_);
+        double minx,miny,maxx,maxy;
+        merc.xyz(x_,y_,z_,minx,miny,maxx,maxy);
+        return box2d<double>(minx,miny,maxx,maxy);
+    }
+
     inline box2d<double> tile_datasource::envelope() const
     {
         if (!extent_initialized_)
         {
-            mapnik::vector::spherical_mercator merc(tile_size_);
-            double minx,miny,maxx,maxy;
-            merc.xyz(x_,y_,z_,minx,miny,maxx,maxy);
-            extent_.init(minx,miny,maxx,maxy);
+            extent_ = get_tile_extent();
             extent_initialized_ = true;
         }
         return extent_;
